library("tilingArray")
options(error=recover)

probeLength = 25

# pombe arraySize?

arraySize   = 2560
nrProbes    = arraySize*arraySize

if(!exists("exon_results")) {
  cat("Loading exon_results.rda.\n")
  load("exon_results.rda")
}

# Different names in fasta files!

if(!exists("chrNo")) {
  chromoAccno = paste("ref|NC_001", c(133:148, 224), "|", sep="")
  chrNo = match(exon_results[[6]], chromoAccno)
  stopifnot(!any(is.na(chrNo)))
}

## Watson strand = antisense = "+" = typically plotted above
## Crick strand = sense = "-" = typically plotted below

if(!exists("strand")) {
  cat("Calculating cstart and cend.\n")
  cstart = exon_results[[7]]
  cend   = exon_results[[8]]
  is.antisense = (cend < cstart)    
  strand = factor(c("+", "-")[1 + is.antisense])
  ## swap
  tmp = cstart[is.antisense]
  cstart[is.antisense] =  cend[is.antisense] 
  cend[is.antisense] = tmp
}


if(!exists("whPM")) {
  cat("Calculating whPM: ")
  #is.25mer = ((exon_results[[4]] == probeLength) &    ## alignment length
  #            (exon_results[[6]] == 0) &              ## number of gaps
  #            (exon_results[[7]] == 1) &              ## start of query
  #            (exon_results[[8]] == probeLength))     ## end of query

  is.25mer = ((exon_results[[4]] == probeLength) &
              (exon_results[[3]] == 0) &
              (exon_results[[12]] == probeLength) &
              (exon_results[[13]] == probeLength))

  whPM = which(is.25mer & (exon_results[[10]]==125) )
  whMM = which(is.25mer & (exon_results[[10]]==116) )
    
  ## double-check
  stopifnot(all(cend[c(whPM, whMM)] - cstart[c(whPM, whMM)] == probeLength - 1))
  
  ## MM are always below PM!
  MMIndexFromPM = function(i) i + arraySize
  stopifnot(all(MMIndexFromPM(exon_results[[2]][whPM] %in% exon_results[[2]][whMM])))

  cat("found", length(whPM), "perfect match and", length(whMM),
      "mismatch BLAST hits.\n")
}

## See which ones have multiple hits
PMindex = exon_results[[2]][whPM]
dupProbes = PMindex[duplicated(PMindex)]

## group PMs by chromosome and strand
sp = split(whPM, list(chr=chrNo[whPM], strand=strand[whPM]))

##--------------------------------------------------
## The along-chromosome data in 'probeAnno'
##--------------------------------------------------
if(!exists("probeAnno")) {
  cat("Making probeanno: ")
  probeAnno = new.env()
  for(i in seq(along=sp)) {
    whc = sp[[i]]
    nm  = names(sp)[i]
    cat(nm, "")
    PMindex = exon_results[[2]][whc]
    assign(paste(nm, "start", sep="."),  cstart[whc], envir=probeAnno)
    assign(paste(nm, "end", sep="."),  cend[whc], envir=probeAnno)
    assign(paste(nm, "index", sep="."),  PMindex, envir=probeAnno)
    assign(paste(nm, "unique", sep="."), !(PMindex %in% dupProbes), envir=probeAnno)
  }
  cat("\n")
}

##--------------------------------------------------
## gff
##--------------------------------------------------
if(!exists("gff")) {
  gffFile="saccharomyces_cerevisiae.gff"
  cat("Reading gff: ")
  ## GFF Files
  gff = read.table(gffFile, sep="\t", as.is=TRUE, quote="", header=FALSE, comment.char="#",
    colClasses=c("factor", "factor", "factor", "integer", "integer",
      "factor", "factor", "factor", "character"))
  colnames(gff) = c("seqname", "source", "feature", "start", "end",
            "score", "strand", "frame", "attributes")
  cat("found", nrow(gff), "rows with classes", paste(sapply(gff, class), collapse=" "), ".\n")
  stopifnot(!any(is.na(gff$start)), !any(is.na(gff$end)))
  
  chrSeqname = 
    c("chrI", "chrII", "chrIII", "chrIV",
      "chrV", "chrVI", "chrVII", "chrVIII", "chrIX",
      "chrX", "chrXI", "chrXII", "chrXIII", "chrXIV",
      "chrXV","chrXVI", "chrMito")
}

##--------------------------------------------------
## The per-probe data in 'probeAnno'
##--------------------------------------------------
featNames = c("no_feature", "CDS", "ncRNA", "nc_primary_transcript",
              "rRNA", "snRNA", "snoRNA", "tRNA")
selGff  = which(gff$feature %in% featNames)  
sgff    = gff[selGff, ]

stopifnot(nrow(sgff)>1000)
stopifnot(all(sgff$strand %in% c("+", "-")))  

mseq = match(sgff$seqname, chrSeqname)
stopifnot(!any(is.na(mseq)), !any(mseq<1), !any(mseq>17))

for(hybeType in c("Reverse", "Direct")) {
  cat("-----", hybeType, "-----\n")
  p = lapply(featNames, function(i) character(nrProbes))
  names(p) = featNames
  ## loop over chromosomes and strands
  #for(chr in c(1)) {
  for(chr in 1:length(chrSeqname)) {
    for(s in c("+", "-")) {
      ifeat  = which(sgff$strand == s & mseq == chr)
      cat(chr, s, ": ", length(ifeat), " features\n", sep="")

      k1 = sgff$start[ifeat]
      k2 = sgff$end[ifeat]
      stopifnot(all(k1<=k2))
   
      Name = getAttributeField(sgff$attributes[ifeat], "Name")

      pas = switch(hybeType,
        Reverse = s,
        Direct  = otherStrand(s)
      )
      sta = get(paste(chr, pas, "start", sep="."), envir=probeAnno)
      ind = get(paste(chr, pas, "index", sep="."), envir=probeAnno)
      sel = isWithinInterval(sta+(probeLength-1)/2, k1, k2)
      wh  = which(colSums(sel)==0)
      ## if(length(wh)>0) {
      ##	cat("\nNo probes for i=", ifeat[wh], "\n")
      ##	print(sgff[ifeat[wh], 1:8])
      ## }
      ## print(table(rowSums(sel)))
       
      for(i in seq(along=ifeat)) {
	fi = ifeat[i]
	p[[as.character(sgff$feature[fi])]] [ind[sel[, i]]]  = Name[i]
      } ## for i
    } ## for s
  } ## for chr

  ## now set 'no_feature' to all PMs that are not something else:
  anyFeature = which(0 < rowSums(sapply(p, function(s) s!="")))
  #allProbes = unlist(mget(paste("1", rep(c("+", "-"), 1), "index", sep="."), envir=probeAnno))
  allProbes = unlist(mget(paste(rep(seq(along=chrSeqname), each=2),
    rep(c("+", "-"), length(chrSeqname)), "index", sep="."),
    envir=probeAnno))
  p[["no_feature"]][setdiff(allProbes, anyFeature)] = "no" 
  cat("length(allProbes)=", length(allProbes),
      "length(anyFeature)=", length(anyFeature),
      "no feature=", length(which(p[["no_feature"]]=="no")), "\n")

  for(i in 1:length(p))
    p[[i]] = factor(p[[i]])

  assign(paste("probe", hybeType, sep=""), p, envir=probeAnno)
} ## for hybetype

cat("\n")


save(probeAnno, gff, chrSeqname, file="probeAnno.rda", compress=TRUE)
cat("Finished.\n")
